import java.lang.reflect.*;

/**
   This program computes pow(2.0, 9.0) and pow(2.0, 10.0)
   the hard way, using reflection.

   This program shows that the invoke() method, which
   is declared vararg (with a variable number of arguments),
   can be called using either an array of arguments,
   or with a variable number of arguments.
*/
public class HardSqrt
{
   public static void main(String[] args)
   throws NoSuchMethodException, IllegalAccessException, InvocationTargetException, ClassNotFoundException
   {
      String className = "java.lang.Math";
      String methodName = "pow";

      Class<?> c = Class.forName( className );

      Class<?> firstArgClass = double.class;

      Class<?> secondArgClass = double.class;

      Method m = c.getDeclaredMethod( methodName, firstArgClass, secondArgClass );

      // invoke the pow() method using a (single) array argument.
      Double[] myArgs = { new Double(2.0), new Double(9.0) };
      Object o = m.invoke( null, myArgs );  // an array argument
      double r = ((Double)o).doubleValue();
      System.out.println(  "r = " + r );

      // invoke the pow() method using multiple arguments.
      o = m.invoke( null, new Double(2.0), new Double(10.0) );  // two double arguments
      r = ((Double)o).doubleValue();
      System.out.println(  "r = " + r );
   }
}